/*=============================================================================
    Copyright (c) 2001-2011 Hartmut Kaiser

    Distributed under the Boost Software License, Version 1.0. (See accompanying
    file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
==============================================================================*/

#if !defined(BOOST_SPIRIT_AUTO_NOV_29_2009_0335PM)
#define BOOST_SPIRIT_AUTO_NOV_29_2009_0335PM

#if defined(_MSC_VER)
#pragma once
#endif

#include <boost/spirit/home/qi/auto/auto.hpp>
#include <boost/spirit/home/qi/detail/parse_auto.hpp>

#endif
